package evoHistory;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;

import processUtils.WeightedParsimony;

import io.FamilyParser;
import io.MatrixParser;
import io.NewickParser;
import units.BinaryTree;
import units.GeneFamily;
import units.Matrix;
import units.Node;

public class EvoHistory {

	private static String familyFile;
	private static String matrixFile;
	private static String treeFile;
	private static String outputFile;
	
	public static void main(String[] args){
		
		//set the file names from user input
		familyFile = args[0];
		matrixFile = args[1];
		treeFile = args[2];
		outputFile = args[3];
		
		//read in the family information
		System.out.println("Reading gene family sizes...");
		FamilyParser family = new FamilyParser(familyFile);
		TreeMap<String, GeneFamily> families = family.parse();
		
		//read the matrix
		System.out.println("Reading the matrix...");
		MatrixParser matrixP = new MatrixParser(matrixFile);
		Matrix matrix = matrixP.parse();
		
		//create the tree and set its root
		System.out.println("Reading the newick tree...");
		BinaryTree tree = new BinaryTree();
		NewickParser newick = new NewickParser(treeFile);
		Node root = newick.parse();
		tree.setRoot(root);
		tree.setNewick(newick.getNewickString());
		
		//do weighted parsimony and write output files
		System.out.println("Caluculating ancestral gene family sizes...\nWriting output...");
		WeightedParsimony parsimony = new WeightedParsimony(tree, families, matrix, outputFile);
		parsimony.calculateParsimony();
		
	}
	
}
