package io;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.TreeMap;

import exceptions.FamilyException;

import units.GeneFamily;

public class FamilyParser {
	
	private String fileName;
	private TreeMap<String, GeneFamily> families;
	
	public FamilyParser(String fileName){
		this.fileName = fileName;
		this.families = new TreeMap<String, GeneFamily>();
	}
	
	public TreeMap<String, GeneFamily> parse(){
		try{
			BufferedReader reader = new BufferedReader(new FileReader(this.fileName));
			String line;
			String[] speciesNames = null;
			while((line = reader.readLine()) != null){
				if(line.startsWith("FAMILY")){
					speciesNames = line.split("\\s+");
				}else{
					HashMap<String, Integer> family = new HashMap<String, Integer>();
					String[] split = line.split("\\s+");
					try{
						testInput(speciesNames.length, split.length);
						for(int i=1; i<split.length; i++){
							family.put(speciesNames[i], Integer.parseInt(split[i]));
						}
						this.families.put(split[0], new GeneFamily(family));
					}catch(FamilyException e){
						System.err.println(e.toString());
						e.printStackTrace();
						System.exit(1);
					}
				}
			}
		}catch(IOException e){
			System.err.println("Error: "+e.getMessage());
			e.printStackTrace();
		}
		return this.families;
	}
	
	private void testInput(int species, int families) throws FamilyException{
		if(species != families){
			throw new FamilyException(species, families);
		}
	}

}
