package io;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import exceptions.NewickException;

import units.Node;

public class NewickParser {

	private String treeString;
	private Vector<String> nodeNames;
	private int internalCounter;
	
	public NewickParser(String fileName){
		String treeString = readTree(fileName);
		try{
			testInput(treeString);
			this.treeString = treeString.replaceAll("\\;", "");
			this.nodeNames = new Vector<String>();
			this.internalCounter = 0;
		}catch(NewickException e){
			System.err.println(e.toString());
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	public void setNewickString(String treeString){
		this.treeString = treeString;
	}
	public String getNewickString(){
		return this.treeString;
	}
	
	private String readTree(String fileName){
		String tree = "";
		try{
			BufferedReader reader = new BufferedReader(new FileReader(fileName));
			String line;
			while((line = reader.readLine()) != null){
				tree = line;
			}
		}catch(IOException e){
			System.out.println("Error: "+e.getMessage());
			e.printStackTrace();
		}
		return tree;
	}
	
	public void setTreeString(String treeString){
		this.treeString = treeString.replaceAll("\\;", "");
	}
	
	public Node parse(){
		int x = this.treeString.lastIndexOf(':');
		Node node = new Node(Double.parseDouble(this.treeString.substring(x+1)));
		String nodeName = parseName(this.treeString, x);
		node.setName(nodeName);
		return build(this.treeString, node, 0, x);
	}
	
	public Node build(String s, Node parent, int from, int to) {
		
		if (s.charAt(from) != '(') {
			return parent;
		}
 
		int b = 0; // bracket counter
		int colon = 0; // colon marker
		int x = from; // position marker
 
		for (int i = from; i < to; i++) {
			char c = s.charAt(i);
 
			if (c == '(')
				b++;
			else if (c == ')')
				b--;
			else if (c == ':')
				colon = i;
 
			if (b == 0 || b == 1 && c == ',') {
				if(s.charAt(i-1) != ')' && (colon+1) < i){
					String branchLength = s.substring(colon+1, i);
					String[] split = branchLength.split("\\)");
					branchLength = split[0];
					String nodeName = parseName(s, colon);
					if(!this.nodeNames.contains(nodeName)){
						Node node = new Node(Double.parseDouble(branchLength));
						node.setName(nodeName);
						
						parent.addDescendent(build(s, node, x + 1, colon));
						x = i;
						this.nodeNames.add(nodeName);
					}
				}
			}
		}
 
		return parent;
	}
	
	private String parseName(String s, int end){
		String name = "node"+this.internalCounter;
		for(int i = end; i >= 0; i--){
			if(s.charAt(i) == ')' || s.charAt(i) == '(' || s.charAt(i) == ','){
				if((i+1) < end){
					name = s.substring(i+1, end);
					String[] split = name.split("\\:");
					return split[0];
				}else{
					return name;
				}
			}
		}
		this.internalCounter++;
		return name;
	}
	
	private void testInput(String treeString) throws NewickException{
		Pattern pattern = Pattern.compile("[a-zA-z.]+:[0-9.]+");
		Matcher match = pattern.matcher(treeString);
		if(!match.find()){
			throw new NewickException(treeString);
		}
	}
}