package io;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import units.BinaryTree;
import units.Node;

public class ResultOutput {
	
	private static String descSuffix = "_desc.txt";
	private static String treeSuffix = "_tree.ph";
	private String descriptionFile;
	private String treeFile;
	
	/**
	 * Constructor creates the file names and deletes the files
	 * if they already exist
	 * @param outputPrefix the prefix for the ouput files
	 */
	public ResultOutput(String outputPrefix){
		this.descriptionFile = outputPrefix+descSuffix;
		if(fileExists(this.descriptionFile)){
			deleteFile(this.descriptionFile);
		}
		this.treeFile = outputPrefix+treeSuffix;
		if(fileExists(this.treeFile)){
			deleteFile(this.treeFile);
		}
	}
	
	public void outputResult(BinaryTree tree, String familyID){
		if(!fileExists(this.descriptionFile)){
			printHeaderLine(tree);
		}
		printFamilyDescription(tree, familyID);
		printFamilyTree(tree, familyID);
	}
	
	private void printFamilyDescription(BinaryTree tree, String familyID){
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(this.descriptionFile, true));
			writer.write(familyID);
			for(Node node : tree.getPreorder(tree.getRoot())){
				writer.write("\t"+node.getFamilySize());
			}
			writer.write("\n");
			writer.close();
		}catch(IOException e){
			System.err.println("Error: "+e.getMessage());
			e.printStackTrace();
		}
	}
	
	private void printFamilyTree(BinaryTree tree, String familyID){
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(this.treeFile, true));
			writer.write(familyID+"\n");
			String newick = tree.getNewick();
			for(Node node : tree.getPreorder(tree.getRoot())){
				if(!node.isLeaf()){
					newick = newick.replaceAll(node.getName(), Integer.toString(node.getFamilySize()));
				}
			}
			writer.write(newick+"\n");
			writer.close();
		}catch(IOException e){
			System.err.println("Error: "+e.getMessage());
			e.printStackTrace();
		}
	}
	
	private void printHeaderLine(BinaryTree tree){
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(this.descriptionFile));
			writer.write("FAMILY");
			for(Node node : tree.getPreorder(tree.getRoot())){
				writer.write("\t"+node.getName());
			}
			writer.write("\n");
			writer.close();
		}catch(IOException e){
			System.err.println("Error: "+e.getMessage());
			e.printStackTrace();
		}
	}
	
	private boolean fileExists(String fileName){
		File file = new File(fileName);
		return file.exists();
	}
	
	private void deleteFile(String fileName){
		File file = new File(fileName);
		file.delete();
	}
}
